==> plotly3d-0.3.8.tar.gz: plotly3d-0.3.8/plotly3d/plot.py <==
import plotly.graph_objects as go
from sklearn.preprocessing import MinMaxScaler
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib

def scatter(points, colors=None, **kwargs):
    """
    Plots 3D scatter plot with optional rescaling, coloring, and customization.
    
    Parameters:
    - points: Array of points to plot.
    - colors: Optional array of colors for each point.
    - scaler: Optional scaler object to rescale points. If None and rescaling is enabled, MinMaxScaler is used.
    - **kwargs: Additional optional arguments:
        - s (float): Size of the markers.
        - alpha (float): Opacity of the markers.
        - force_continuous (bool): When True, applies a continuous colormap but discretizes it for categorical data.
        - title (str): Title of the plot.
        - filename (str): If provided, saves the plot to this file.
        - rescale (bool): If True, rescales points using the provided or default scaler.
        - fig (go.Figure): Plotly figure object to which the scatter plot will be added. If None, a new figure is created.
        - xtitle (str), ytitle (str), ztitle (str): Titles for the X, Y, and Z axes.
        - colorscale (str): Colormap to use ('Viridis', 'Inferno', etc.).
        - legend (bool): Whether to show legend.
    """
    is_3d = points.shape[1] == 3
    plot_func = go.Scatter3d if is_3d else go.Scatter
    scaler = kwargs.get('scaler', None)
    s = kwargs.get('s', 6)  # Default size
    alpha = kwargs.get('alpha', 0.8)  # Default opacity
    title = kwargs.get('title', 'Plot')
    filename = kwargs.get('filename', None)
    xtitle = kwargs.get('xtitle', 'X')
    ytitle = kwargs.get('ytitle', 'Y')
    ztitle = kwargs.get('ztitle', 'Z')
    force_continuous = kwargs.get('force_continuous', False)
    rescale = kwargs.get('rescale', True)
    legend = kwargs.get('legend', True)
    colorscale = kwargs.get('colorscale', 'Viridis').capitalize()  # Ensure correct case
    fig = kwargs.get('fig', go.Figure())
    points = np.asarray(points)
    colors = np.asarray(colors) if colors is not None else None
    
    if rescale:
        if scaler is None:
            scaler = MinMaxScaler()
            scaler.fit(points)
        points_s = scaler.transform(points)
    else:
        points_s = points

    if colors is None:
        colors = np.zeros(points.shape[0])
        is_categorical = True
    else:
        unique_colors = np.unique(colors)
        is_categorical = not force_continuous and (len(unique_colors) / len(colors) < 0.05)
    
    if force_continuous and colors is not None and is_categorical:
        # Normalize color values for continuous colormap application
        color_map, _ = pd.factorize(colors, sort=True)
        color_vals = MinMaxScaler().fit_transform(color_map.reshape(-1, 1)).flatten()
        cmap = plt.get_cmap(colorscale)
        discrete_mapping = cmap(color_vals)  # Apply colormap
        
        hex_colors = [matplotlib.colors.rgb2hex(color) for color in discrete_mapping]  # Convert to hex

        fig.add_trace(plot_func(
            x=points_s[:, 0],
            y=points_s[:, 1],
            z=points_s[:, 2] if is_3d else None,
            mode='markers',
            marker=dict(size=s, color=hex_colors, opacity=alpha),
            hoverinfo='text',
            text=colors,  # Show original category names on hover
        ))
    else:
        # Continuous or non-force_continuous categorical handling
        if is_3d:
            fig.add_trace(plot_func(
                x=points_s[:, 0],
                y=points_s[:, 1],
                z=points_s[:, 2],
                mode='markers',
                marker=dict(size=s, color=colors, colorscale=colorscale, opacity=alpha, colorbar=dict(title='Color Scale') if not is_categorical else None),
            ))
        else:
            fig.add_trace(plot_func(
                x=points_s[:, 0],
                y=points_s[:, 1],
                mode='markers',
                marker=dict(size=s, color=colors, colorscale=colorscale, opacity=alpha, colorbar=dict(title='Color Scale') if not is_categorical else None),
            ))

    # Layout updates
    if is_3d:
        fig.update_layout(
            title=title,
            scene=dict(
                xaxis_title=xtitle,
                yaxis_title=ytitle,
                zaxis_title=ztitle
            ),
            showlegend=legend
        )
    else:
        fig.update_layout(
            title=title,
            xaxis_title=xtitle,
            yaxis_title=ytitle,
            showlegend=legend
        )
    if filename is not None:
        fig.write_html(filename)

    return fig

# for compatability with previous versions
plot_3d = scatter
scatter3d = scatter

# def trajectories(trajs, s=1, alpha=1, title="3D Plot", filename=None, rescale=True, fig=None, xtitle='X', ytitle='Y', ztitle='Z', scaler=None, legend_label="Trajectories"):
def trajectories(trajs, **kwargs):
    """
    Plots trajectories in 3D space using Plotly, with thinner gray lines without markers and reduced line opacity.

    Parameters:
    - trajs: numpy array of shape (a, b, c), where
      a: time bins, b: trajectory index, c: space dimension (c=3 for 3D).

    Returns:
    - Plotly figure containing the trajectories plotted in 3D space.
    """
    is_3d = trajs.shape[2] == 3
    plot_func = go.Scatter3d if is_3d else go.Scatter
    s = kwargs.get('s', 1)
    s_end = kwargs.get('s_end', 1)
    alpha = kwargs.get('alpha', 1)
    title = kwargs.get('title', 'Plot')
    filename = kwargs.get('filename', None)
    rescale = kwargs.get('rescale', True)
    fig = kwargs.get('fig', go.Figure())
    xtitle = kwargs.get('xtitle', 'X')
    ytitle = kwargs.get('ytitle', 'Y')
    ztitle = kwargs.get('ztitle', 'Z')
    scaler = kwargs.get('scaler', None)
    legend_label = kwargs.get('legend_label', 'Trajectories')

    trajs = np.asarray(trajs)
    if rescale:
        if scaler is None:
            scaler = MinMaxScaler()
            scaler.fit(trajs.reshape(-1, trajs.shape[2]))
        trajs_s = scaler.transform(trajs.reshape(-1, trajs.shape[2])).reshape(trajs.shape)
    else:
        trajs_s = trajs

    # Number of trajectories
    num_trajectories = trajs.shape[1]
    
    # Use a legendgroup and make the dummy trace for the legend entry
    legendgroup = "trajectories"
    # Add a dummy trace for the legend entry, enabling its visibility toggles the group
    if is_3d:
        fig.add_trace(plot_func(
            x=[None],
            y=[None],
            z=[None],
            mode='lines',
            line=dict(color='gray', width=s),
            name=legend_label,
            legendgroup=legendgroup,
            # This is the key for toggling visibility
            visible=True
        ))
    else:
        fig.add_trace(plot_func(
            x=[None],
            y=[None],
            mode='lines',
            line=dict(color='gray', width=s),
            name=legend_label,
            legendgroup=legendgroup,
            # This is the key for toggling visibility
            visible=True
        ))
    # Iterate over each trajectory and plot it, linked by legendgroup
    for i in range(num_trajectories):
        trajectory = trajs_s[:, i, :]
        if is_3d:
            fig.add_trace(plot_func(
                x=trajectory[:, 0],
                y=trajectory[:, 1],
                z=trajectory[:, 2],
                mode='lines',
                line=dict(width=s, color='gray'),
                opacity=alpha,
                showlegend=False,  # Do not show these individual traces in the legend
                legendgroup=legendgroup,  # Link visibility control to the dummy trace
                visible=True  # Initially visible
            ))
        else:
            fig.add_trace(plot_func(
                x=trajectory[:, 0],
                y=trajectory[:, 1],
                mode='lines',
                line=dict(width=s, color='gray'),
                opacity=alpha,
                showlegend=False,  # Do not show these individual traces in the legend
                legendgroup=legendgroup,  # Link visibility control to the dummy trace
                visible=True  # Initially visible
            ))
        # Add endpoint marker
        if is_3d:
            fig.add_trace(plot_func(
                x=[trajectory[-1, 0]],
                y=[trajectory[-1, 1]],
                z=[trajectory[-1, 2]],
                mode='markers',
                marker=dict(size=s_end, color='red'), # Customize color and size as needed
                showlegend=False,
                legendgroup=legendgroup,
                visible=True
            ))
        else:
            fig.add_trace(plot_func(
                x=[trajectory[-1, 0]],
                y=[trajectory[-1, 1]],
                mode='markers',
                marker=dict(size=s_end, color='red'), # Customize color and size as needed
                showlegend=False,
                legendgroup=legendgroup,
                visible=True
            ))

    # Set the layout for the 3D plot
    if is_3d:
        fig.update_layout(
            title=title,
            scene=dict(
                xaxis_title=xtitle, yaxis_title=ytitle, zaxis_title=ztitle,
            ),
            margin=dict(l=0, r=0, b=0, t=30)
        )
    else:
        fig.update_layout(
            title=title,
            xaxis_title=xtitle, yaxis_title=ytitle,
            margin=dict(l=0, r=0, b=0, t=30)
        )
    if filename is not None:
        fig.write_html(filename)

    return fig

==> plotly3d-0.4.0.tar.gz: plotly3d-0.4.0/plotly3d/plot.py <==
import plotly.graph_objects as go
from sklearn.preprocessing import MinMaxScaler
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib

def scatter(points, colors=None, **kwargs):
    # FIXME: debug force_continous!!
    """
    Plots 3D scatter plot with optional rescaling, coloring, and customization.
    
    Parameters:
    - points: Array of points to plot.
    - colors: Optional array of colors for each point.
    - scaler: Optional scaler object to rescale points. If None and rescaling is enabled, MinMaxScaler is used.
    - **kwargs: Additional optional arguments:
        - s (float): Size of the markers.
        - alpha (float): Opacity of the markers.
        - force_continuous (bool): When True, applies a continuous colormap but discretizes it for categorical data.
        - title (str): Title of the plot.
        - filename (str): If provided, saves the plot to this file.
        - rescale (bool): If True, rescales points using the provided or default scaler.
        - fig (go.Figure): Plotly figure object to which the scatter plot will be added. If None, a new figure is created.
        - xtitle (str), ytitle (str), ztitle (str): Titles for the X, Y, and Z axes.
        - colorscale (str): Colormap to use ('Viridis', 'Inferno', etc.).
        - legend (bool): Whether to show legend.
    """
    is_3d = points.shape[1] == 3
    plot_func = go.Scatter3d if is_3d else go.Scatter
    scaler = kwargs.get('scaler', None)
    s = kwargs.get('s', 6)  # Default size
    alpha = kwargs.get('alpha', 0.8)  # Default opacity
    title = kwargs.get('title', 'Plot')
    filename = kwargs.get('filename', None)
    xtitle = kwargs.get('xtitle', 'X')
    ytitle = kwargs.get('ytitle', 'Y')
    ztitle = kwargs.get('ztitle', 'Z')
    force_continuous = kwargs.get('force_continuous', False)
    rescale = kwargs.get('rescale', False)
    legend = kwargs.get('legend', True)
    colorscale = kwargs.get('colorscale', 'Viridis').capitalize()  # Ensure correct case
    fig = kwargs.get('fig', go.Figure())
    points = np.asarray(points)
    colors = np.asarray(colors) if colors is not None else None
    
    if rescale:
        if scaler is None:
            scaler = MinMaxScaler()
            scaler.fit(points)
        points_s = scaler.transform(points)
    else:
        points_s = points

    if colors is None:
        colors = np.zeros(points.shape[0])
        is_categorical = True
    else:
        unique_colors = np.unique(colors)
        is_categorical = not force_continuous and (len(unique_colors) / len(colors) < 0.05)
    
    if force_continuous and colors is not None and is_categorical:
        # Normalize color values for continuous colormap application
        color_map, _ = pd.factorize(colors, sort=True)
        color_vals = MinMaxScaler().fit_transform(color_map.reshape(-1, 1)).flatten()
        cmap = plt.get_cmap(colorscale)
        discrete_mapping = cmap(color_vals)  # Apply colormap
        
        hex_colors = [matplotlib.colors.rgb2hex(color) for color in discrete_mapping]  # Convert to hex

        fig.add_trace(plot_func(
            x=points_s[:, 0],
            y=points_s[:, 1],
            z=points_s[:, 2] if is_3d else None,
            mode='markers',
            marker=dict(size=s, color=hex_colors, opacity=alpha),
            hoverinfo='text',
            text=colors,  # Show original category names on hover
        ))
    else:
        # Continuous or non-force_continuous categorical handling
        if is_3d:
            fig.add_trace(plot_func(
                x=points_s[:, 0],
                y=points_s[:, 1],
                z=points_s[:, 2],
                mode='markers',
                marker=dict(size=s, color=colors, colorscale=colorscale, opacity=alpha, colorbar=dict(title='Color Scale') if not is_categorical else None),
            ))
        else:
            fig.add_trace(plot_func(
                x=points_s[:, 0],
                y=points_s[:, 1],
                mode='markers',
                marker=dict(size=s, color=colors, colorscale=colorscale, opacity=alpha, colorbar=dict(title='Color Scale') if not is_categorical else None),
            ))

    # Layout updates
    if is_3d:
        fig.update_layout(
            title=title,
            scene=dict(
                xaxis_title=xtitle,
                yaxis_title=ytitle,
                zaxis_title=ztitle
            ),
            showlegend=legend
        )
    else:
        fig.update_layout(
            title=title,
            xaxis_title=xtitle,
            yaxis_title=ytitle,
            showlegend=legend
        )
    if filename is not None:
        fig.write_html(filename)

    return fig

# for compatability with previous versions
plot_3d = scatter
scatter3d = scatter

# def trajectories(trajs, s=1, alpha=1, title="3D Plot", filename=None, rescale=True, fig=None, xtitle='X', ytitle='Y', ztitle='Z', scaler=None, legend_label="Trajectories"):
def trajectories(trajs, **kwargs):
    """
    Plots trajectories in 3D space using Plotly, with thinner gray lines without markers and reduced line opacity.

    Parameters:
    - trajs: numpy array of shape (a, b, c), where
      a: time bins, b: trajectory index, c: space dimension (c=3 for 3D).

    Returns:
    - Plotly figure containing the trajectories plotted in 3D space.
    """
    is_3d = trajs.shape[2] == 3
    plot_func = go.Scatter3d if is_3d else go.Scatter
    s = kwargs.get('s', 1)
    s_end = kwargs.get('s_end', 1)
    alpha = kwargs.get('alpha', 1)
    title = kwargs.get('title', 'Plot')
    filename = kwargs.get('filename', None)
    rescale = kwargs.get('rescale', False)
    fig = kwargs.get('fig', go.Figure())
    xtitle = kwargs.get('xtitle', 'X')
    ytitle = kwargs.get('ytitle', 'Y')
    ztitle = kwargs.get('ztitle', 'Z')
    scaler = kwargs.get('scaler', None)
    legend_label = kwargs.get('legend_label', 'Trajectories')
    c = kwargs.get('c', 'gray')
    ctip = kwargs.get('ctip', 'red')

    trajs = np.asarray(trajs)
    if rescale:
        if scaler is None:
            scaler = MinMaxScaler()
            scaler.fit(trajs.reshape(-1, trajs.shape[2]))
        trajs_s = scaler.transform(trajs.reshape(-1, trajs.shape[2])).reshape(trajs.shape)
    else:
        trajs_s = trajs

    # Number of trajectories
    num_trajectories = trajs.shape[1]
    
    # Use a legendgroup and make the dummy trace for the legend entry
    legendgroup = "trajectories"
    # Add a dummy trace for the legend entry, enabling its visibility toggles the group
    if is_3d:
        fig.add_trace(plot_func(
            x=[None],
            y=[None],
            z=[None],
            mode='lines',
            line=dict(color=c, width=s),
            name=legend_label,
            legendgroup=legendgroup,
            # This is the key for toggling visibility
            visible=True
        ))
    else:
        fig.add_trace(plot_func(
            x=[None],
            y=[None],
            mode='lines',
            line=dict(color=c, width=s),
            name=legend_label,
            legendgroup=legendgroup,
            # This is the key for toggling visibility
            visible=True
        ))
    # Iterate over each trajectory and plot it, linked by legendgroup
    for i in range(num_trajectories):
        trajectory = trajs_s[:, i, :]
        if is_3d:
            fig.add_trace(plot_func(
                x=trajectory[:, 0],
                y=trajectory[:, 1],
                z=trajectory[:, 2],
                mode='lines',
                line=dict(width=s, color=c),
                opacity=alpha,
                showlegend=False,  # Do not show these individual traces in the legend
                legendgroup=legendgroup,  # Link visibility control to the dummy trace
                visible=True  # Initially visible
            ))
        else:
            fig.add_trace(plot_func(
                x=trajectory[:, 0],
                y=trajectory[:, 1],
                mode='lines',
                line=dict(width=s, color=c),
                opacity=alpha,
                showlegend=False,  # Do not show these individual traces in the legend
                legendgroup=legendgroup,  # Link visibility control to the dummy trace
                visible=True  # Initially visible
            ))
        # Add endpoint marker
        if is_3d:
            fig.add_trace(plot_func(
                x=[trajectory[-1, 0]],
                y=[trajectory[-1, 1]],
                z=[trajectory[-1, 2]],
                mode='markers',
                marker=dict(size=s_end, color=ctip), # Customize color and size as needed
                showlegend=False,
                legendgroup=legendgroup,
                visible=True
            ))
        else:
            fig.add_trace(plot_func(
                x=[trajectory[-1, 0]],
                y=[trajectory[-1, 1]],
                mode='markers',
                marker=dict(size=s_end, color=ctip), # Customize color and size as needed
                showlegend=False,
                legendgroup=legendgroup,
                visible=True
            ))

    # Set the layout for the 3D plot
    if is_3d:
        fig.update_layout(
            title=title,
            scene=dict(
                xaxis_title=xtitle, yaxis_title=ytitle, zaxis_title=ztitle,
            ),
            margin=dict(l=0, r=0, b=0, t=30)
        )
    else:
        fig.update_layout(
            title=title,
            xaxis_title=xtitle, yaxis_title=ytitle,
            margin=dict(l=0, r=0, b=0, t=30)
        )
    if filename is not None:
        fig.write_html(filename)

    return fig
